using System.Text.Json;
using System.Text.Json.Serialization;
using System.Text.Json.Nodes;
using System.Threading.Tasks;

namespace Example;

public abstract class TestServiceServer<TContext> : JRpcService<TContext> {

   public override string Name {
      get {
         return "TestService";
      }
   }

   public TestServiceServer() {
      this.RegisterFunction("AddValuesSingleParam");
      this.RegisterFunction("AddValuesMultipleParams");
      this.RegisterFunction("ReturningVoid");
      this.RegisterFunction("OnEvent");
      this.RegisterFunction("ThrowingError");
      this.RegisterFunction("FunctionWithArrayAsParamAndReturn");
   }

   public abstract Task<AddValueResponse> AddValuesSingleParam(AddValueRequest request, TContext ctx);
   public abstract Task<double> AddValuesMultipleParams(double value1, double value2, TContext ctx);
   public abstract Task ReturningVoid(double param1, TContext ctx);
   public abstract void OnEvent(string param1, TContext ctx);
   public abstract Task ThrowingError(TContext ctx);
   public abstract Task<IList<double>> FunctionWithArrayAsParamAndReturn(List<double> values1, List<double> values2, TContext ctx);

   public async override Task<JsonNode?> HandleRequest(string func, JsonNode param, TContext context) {
      switch(func) {
         case "AddValuesSingleParam": {
            if(param is JsonObject) {
               var ja = new JsonArray(param["request"]);
               param = ja;
            }
            var result = await this.AddValuesSingleParam(param[0]!.Deserialize<AddValueRequest>(), context);
            return JsonSerializer.SerializeToNode(result);
         }

         case "AddValuesMultipleParams": {
            if(param is JsonObject) {
               var ja = new JsonArray(param["value1"], param["value2"]);
               param = ja;
            }
            var result = await this.AddValuesMultipleParams(param[0]!.Deserialize<double>(), param[1]!.Deserialize<double>(), context);
            return JsonSerializer.SerializeToNode(result);
         }

         case "ReturningVoid": {
            if(param is JsonObject) {
               var ja = new JsonArray(param["param1"]);
               param = ja;
            }
            await this.ReturningVoid(param[0]!.Deserialize<double>(), context);
            return null;
         }

         case "OnEvent": {
            if(param is JsonObject) {
               var ja = new JsonArray(param["param1"]);
               param = ja;
            }
            this.OnEvent(param[0]!.Deserialize<string>(), context);
            return null;
         }

         case "ThrowingError": {
            if(param is JsonObject) {
               var ja = new JsonArray();
               param = ja;
            }
            await this.ThrowingError(context);
            return null;
         }

         case "FunctionWithArrayAsParamAndReturn": {
            if(param is JsonObject) {
               var ja = new JsonArray(param["values1"], param["values2"]);
               param = ja;
            }
            var result = await this.FunctionWithArrayAsParamAndReturn(param[0]!.Deserialize<List<double>>(), param[1]!.Deserialize<List<double>>(), context);
            return JsonSerializer.SerializeToNode(result);
         }

         default:
            throw new Exception("Invalid Method!");
      }
   }
}